function [drawrarc]=drawrarc(r,xstart,xstop,linewidth,linestyle,linecolor,markersize);
% drawrarc
% drawrarc(r,xstart,xstop,linewidth,linestyle,linecolor,markersize)
% Draws one or several constant-resistance arcs in the current (Smith) graph
%
% Parameters (column vector in case of multiple arcs):
% r - normalized resistance
% xstart - start reactance (drawrarc always draws counterclockwise)
% xstop - stop reactance
%
% Optional:
% linewidth - in 1/72 inch (points)
% linestyle - MATLAB parameter Ex. '-', '-.' etc
% linecolor - [R,G,B]
% markersize - in 1/72 inch (points)
% 
% drawrarc returns a handle to the objects drawn. In case of
% multiple arcs a vertical vector is returned containing the
% handles.


% 1996-12-18 Written by Lars Sundstrom,
% Dept of Appl Electronics, Lund University
% Copyright (c) 1996. All rights reserved.


% Global variables for parameters that are not specified
global daelibinitialized
global daecircleresolution
global daelinewidth
global daelinestyle
global daelinecolor
global daemarkersize

% Define global variables if necessary
if isempty(daelibinitialized),
	initdae;
end

% Set undefined parameters 
if nargin==3,
	linecolor=daelinecolor;
	linestyle=daelinestyle;
	linewidth=daelinewidth;
	markersize=daemarkersize;
else
	if nargin<7 markersize=daemarkersize; end
	if nargin<6 linecolor=daelinecolor; end
	if nargin<5 linestyle=daelinestyle; end
end

% Number of arcs
c=length(r);

% storage for handles
h=zeros(c,1);

if c==1

	if xstart<xstop,
		tmp=xstart;
		xstart=xstop;
		xstop=tmp;
	end

	center=r/(r+1);
	radius=1/(r+1);
	startangle=180*angle(nz2g(r+i*xstart)-center)/pi;
	stopangle=180*angle(nz2g(r+i*xstop)-center)/pi;
	
	h(1,1)=drawarc([center,radius,startangle,stopangle],linewidth,linestyle,linecolor,markersize);
else

	cc=size(linewidth);
	if cc(1,1)==1, linewidth=ones(c,1)*linewidth; end

	cc=size(linestyle);
	if cc(1,1)==1, linestyle=setstr(ones(c,1)*linestyle); end
	
	cc=size(linecolor);
	if cc(1,1)==1, linecolor=ones(c,1)*linecolor; end

	cc=size(markersize);
	if cc(1,1)==1, markersize=ones(c,1)*markersize; end

	for k=1:c,

		if xstart(k,1)<xstop(k,1),
			tmp=xstart(k,1);
			xstart(k,1)=xstop(k,1);
			xstop(k,1)=tmp;
		end

		center=r(k,1)/(r(k,1)+1);
		radius=1/(r(k,1)+1);
		startangle=180*angle(nz2g(r(k,1)+i*xstart(k,1))-center)/pi;
		stopangle=180*angle(nz2g(r(k,1)+i*xstop(k,1))-center)/pi;
		h(k,1)=drawarc([center,radius,startangle,stopangle],linewidth(k,1),linestyle(k,:),linecolor(k,:),markersize(k,:));
	end
end

drawrarc=h;

