function [smtool]=smtool(action,flag);
% smtool
% smtool (without any arguments) creates a window with a Smith
% chart. The arguments "action" and "flag"
% are for internal use only.
% Functions(buttons):
% Clear All     - Clears chart completely, including Smith chart grid
% new Z grid    - Creates a new Z-Smith chart grid
% new Y grid    - Creates a new Y-Smith chart grid
% Collect       - Click in chart to collect points given as refl. coeff.
%                 When finished press the Return key. Collected points
%                 are stored in the global variable smtoolcollect
%                 (you must declare this variable as global before using
%                 it ("global smtoolcollect OR you can use the smval
%                 function.)
% In            - Zoom in by a factor of two
% N             - Return to normal scale
% Out           - Zoom out by a factor of two
% Move Origin   - Click in the chart to move origin 
% Delete Object - Click on object to delete it from the chart. Includes
%                 Smith chart grid lines and text objects.
% Close         - Closes the window.
% Place Text    - To place a text in the graph - enter the text into the
%                 edit field, press the "Place Text" button and then click
%                 in the graph to position the text.


% 1994-12-01 Written by Lars Sundstrom,
% Dept of Appl Electronics, Lund University
% Copyright (c) 94-96. All rights reserved.
% 1994-12-01: written by Lars Sundstrom, Appl. Electronics, Lund University
% 1996-10-16: moved buttons and text fields to the right of the plot
% to avoid an offset in the print-out. Fixed a bug that made the window
% beep whenever th pointer was within the plot and another window was active.
% 1998-05-22: cleaned up graphical appearance and made it compatible with
% MATLAB 5.x.
% 2014-12-03 Written by Johan Wernehag, EIT, Lund University. Function
% DrawMode not supported in release 2014b, replaced by SortMethod. Other
% functions in mouse motion detection changed.


% Globals
global  smtoolcollect;	% This variable is used to store points collected by the user


% Constants

% Index to links in userdata
protectindex=1;
absGHindex=2;
argGHindex=3;
resiHindex=4;
reacHindex=5;
condHindex=6;
suscHindex=7;
swrHindex=8;
editTextHindex=9;
cursorStateindex=10;


arrow=1;
crosshair=10;

% Rectangle for the plot
plotRect=[0.08 0.13 0.68 0.85];

% If no arguments then start initialization 
if nargin < 1
    action = 'initiate';
end


% Start long "case action of..." statement

if strcmp(action,'initiate'),			% ********* CASE INITIATE *********

	% Begin initialization procedure
 	
	% Define rectangles for the buttons
	offset=[0 0.05 0 0];					% Position offset between buttons
	goffset=[0 0.065 0 0];					% Position offset between groups of buttons
	clearAllBtnRect=[.77 .93 .21 .05];
	newZBtnRect=clearAllBtnRect-goffset;
	newYBtnRect=newZBtnRect-offset;
	collectBtnRect=newYBtnRect-goffset;
	zoomInBtnRect=collectBtnRect-goffset;
	offsetBtnRect=zoomInBtnRect-offset;
	deleteBtnRect=offsetBtnRect-goffset;
	zoomInBtnRect(:,3)=0.07;
	normalZoomBtnRect=zoomInBtnRect+[0.07 0 0 0];
	zoomOutBtnRect=normalZoomBtnRect+[0.07 0 0 0];
	closeBtnRect=[.77 .02 .21 .05];
	placeTextBtnRect=[plotRect(:,1) , 0.02 , .21 , 0.05];
	editTextRect=placeTextBtnRect+[0.22 , 0 , 1-placeTextBtnRect(:,1)-placeTextBtnRect(:,3)-0.46, 0];
	
	% Define rectangles for the text fields	
	offset=[0 0.05 0 0];					% Position offset between adjacent text fields
	absGRect=[.77 .45 .21 .05];				% Text field for magnitude of refl. coeff.
	argGRect=absGRect-offset;				% Text field rect. for argument of refl. coeff.
	resiRect=argGRect-offset;				% Text field rect. for normalized resistance
	reacRect=resiRect-offset;				% Text field rect. for normalized reactance
	condRect=reacRect-offset;				% Text field rect. for normalized conductance
	suscRect=condRect-offset;				% Text field rect. for normalized susceptance
	swrRect=suscRect-offset;				% Text field rect. for standing-wave-ratio


	% Create figure window
   	figH = figure;
      set(figH,'Units','Normalized','BackingStore','on');
   	smtool=figH;	% Return handle to user

	% CREATE OBJECTS - ALL OBJECTS WILL HAVE USERDATA(1,1)=1 TO MARK THEM AS BELONGING
	% TO THE WINDOW SO THAT THEY CANNOT BE DELETED WITH THE DELETE FUNCTION.  
	% Create buttons
	clearAllBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',clearAllBtnRect,...
    	'Callback','smtool(''clearall'')','String','Clear All',...
    	'UserData',1);
	newZBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',newZBtnRect,...
    	'Callback','smtool(''newzgrid'')','String','New Z Grid',...
    	'UserData',1);
	newYBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',newYBtnRect,...
    	'Callback','smtool(''newygrid'')','String','New Y Grid',...
    	'UserData',1);
	collectBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',collectBtnRect,...
    	'Callback','smtool(''collect'')','String','Collect',...
    	'UserData',1);
	zoomInBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',zoomInBtnRect,...
    	'Callback','smtool(''zoomin'')','String','In',...
    	'UserData',1);
	normalzoomBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',normalZoomBtnRect,...
    	'Callback','smtool(''nozoom'')','String','N',...
    	'UserData',1);
	zoomOutBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',zoomOutBtnRect,...
    	'Callback','smtool(''zoomout'')','String','Out',...
    	'UserData',1);
	offsetBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',offsetBtnRect,...
    	'Callback','smtool(''offset'')','String','Move Origin...',...
    	'UserData',1);
	deleteBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',deleteBtnRect,...
    	'Callback','smtool(''delete'')','String','Delete Object...',...
   		'UserData',1);
	CloseBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',closeBtnRect,...
    	'Callback','smtool(''done'')','String','Close',...
   		'UserData',1);
	placeTextBtnH = uicontrol('Style','Pushbutton','Units','normalized','Position',placeTextBtnRect,...
    	'Callback','smtool(''placetext'')','String','Place Text:',...
   		'UserData',1);

	% Create text fields
	absGH=uicontrol('Style','text','Units','normalized','Position',absGRect,...
        'HorizontalAlignment','left',...
        'String','abs(G)=','UserData',1);
	argGH=uicontrol('Style','text','Units','normalized','Position',argGRect,...
        'HorizontalAlignment','left',...
        'String','arg(G)=','UserData',1);
	resiH=uicontrol('Style','text','Units','normalized','Position',resiRect,...
        'HorizontalAlignment','left',...
        'String','r=','UserData',1);
	reacH=uicontrol('Style','text','Units','normalized','Position',reacRect,...
        'HorizontalAlignment','left',...
        'String','x=','UserData',1);
	condH=uicontrol('Style','text','Units','normalized','Position',condRect,...
        'HorizontalAlignment','left',...
        'String','g=','UserData',1);
	suscH=uicontrol('Style','text','Units','normalized','Position',suscRect,...
        'HorizontalAlignment','left',...
        'String','b=','UserData',1);
	swrH=uicontrol('Style','text','Units','normalized','Position',swrRect,...
        'HorizontalAlignment','left',...
        'String','SWR=','UserData',1);
	editTextH=uicontrol('Style','edit','Units','normalized','Position',editTextRect,...
        'HorizontalAlignment','left',...
        'UserData',1);
       

   % Create background plot
   %plotH =axes('Position',plotRect,'Box','on','DrawMode','fast','UserData',1);
   %Modified due to changes in release 2014b
    plotH = axes('Position',plotRect,'Box','on','SortMethod','depth','UserData',1);
	hold on;
	coord=[-1.15,1.15,-1.15,1.15];
	axis(coord);
	axis('equal');
	drawrci([0,0.2,0.5,1,2,5]');
	drawxci([0.2,0.5,1,2,5]');
	drawxci(-[0.2,0.5,1,2,5]');
	plotc([-1;1]);

	% Attach mouse tracking procedure, also pass handle to figure
    Ver = version('-release');
	if sum(Ver > '2014a') ; % New command after version 2014a
        set(figH,'WindowButtonMotionFcn',['smtool(''motion'',', num2str(figH.Number),')']);
    else
        set(figH,'WindowButtonMotionFcn',['smtool(''motion'',', num2str(figH),')']);
    end
	% Define links between the figure and the objects
	links(protectindex)=1;
	links(absGHindex)=absGH;
	links(argGHindex)=argGH;
	links(resiHindex)=resiH;
	links(reacHindex)=reacH;
	links(condHindex)=condH;
	links(suscHindex)=suscH;
	links(swrHindex)=swrH;
	links(editTextHindex)=editTextH;
	links(cursorStateindex)=arrow;
	set(figH,'UserData',links);



	% End of initialization procedure.

elseif strcmp(action,'motion'),			% ********* CASE MOTION *********

	% Begin mouse tracking procedure
	
	% Get handle to figure from input parameter
	figH=flag;
	plotH=get(figH,'CurrentAxes');

	% Get links to objects
	links=get(figH,'UserData');
	
	% Get point in figure coordinates
	figpoint=get(figH,'CurrentPoint');

    % Check if cursorstate should be changed and whether or not to present data
   	insideplot=(figpoint(1)>plotRect(1,1)) &...
    	(figpoint(1)<plotRect(1,1)+plotRect(1,3)) &...
    	(figpoint(2)>plotRect(1,2)) &...
		(figpoint(2)<plotRect(1,2)+plotRect(1,4)) ;
    
    if insideplot,
   
   		% Get point in plot coordinates
    	plotpoint = get(plotH,'CurrentPoint');

    	% Make sure that we have a crosshair cursor
    	cursorState=links(cursorStateindex);
		if cursorState==arrow,
			% We just went inside the plot - change the cursor and display data
			set(figH,'Pointer','crosshair');
			links(cursorStateindex)=crosshair;
			set(figH,'UserData',links)
		end
		
    	% Convert to quantities to be displayed
    	gamma=plotpoint(1,1)+i*plotpoint(1,2);
    	impedance=(1+gamma)/(1-gamma);
    	admittance=1/impedance;
    	swr=(1+sqrt(gamma*conj(gamma)))/(1-sqrt(gamma*conj(gamma)));
    	set(links(absGHindex),'String',sprintf('abs(G)=%6.3f',sqrt(gamma*conj(gamma))));
    	set(links(argGHindex),'String',sprintf('arg(G)=%6.3f',180*angle(gamma)/pi));
    	set(links(resiHindex),'String',sprintf('r=%6.3f',real(impedance)));
    	set(links(reacHindex),'String',sprintf('x=%6.3f',imag(impedance)));
    	set(links(condHindex),'String',sprintf('g=%6.3f',real(admittance)));
   	 	set(links(suscHindex),'String',sprintf('b=%6.3f',imag(admittance)));
    	set(links(swrHindex),'String',sprintf('SWR=%6.3f',swr));
    else
    	% Make sure that we have an arrow cursor
    	cursorState=links(cursorStateindex);
		if cursorState==crosshair,
			% We just went outside the plot - change the cursor and clear the last read data
			set(figH,'Pointer','arrow');
			links(cursorStateindex)=arrow;
			set(figH,'UserData',links)
    		set(links(absGHindex),'String','abs(G)=');
    		set(links(argGHindex),'String','arg(G)=');
    		set(links(resiHindex),'String','r=');
    		set(links(reacHindex),'String','x=');
    		set(links(condHindex),'String','g=');
   	 		set(links(suscHindex),'String','b=');
    		set(links(swrHindex),'String','SWR=');
		end
    
    end

	% End mouse tracking procedure

elseif strcmp(action,'clearall'),			% ********* CASE CLEARALL *********

	% Begin clear all procedure

	cla;
	
	% End clear all procedure

elseif strcmp(action,'newzgrid'),			% ********* CASE NEWZGRID *********

	% Begin new Z grid procedure

	drawrci([0,0.2,0.5,1,2,5]');
	drawxci([0.2,0.5,1,2,5]');
	drawxci(-[0.2,0.5,1,2,5]');
	plotc([-1,1]');
	
	% End new Z grid procedure

elseif strcmp(action,'newygrid'),			% ********* CASE NEWYGRID *********

	% Begin new Y grid procedure

	drawgci([0,0.2,0.5,1,2,5]');
	drawbci([0.2,0.5,1,2,5]');
	drawbci(-[0.2,0.5,1,2,5]');
	plotc([-1,1]');
	
	% End new Y grid procedure

elseif strcmp(action,'collect'),			% ********* CASE COLLECT *********

	% Begin collection procedure

	% Get handle to current figure
	figH=gcf;
	
	% Get links to objects
	links=get(figH,'UserData');

	% Collect points
	smtoolcollect=ginput;

	% Convert to complex quantities
	smtoolcollect=smtoolcollect(:,1)+i*smtoolcollect(:,2);

	% End collection procedure

elseif strcmp(action,'delete'),			% ********* CASE DELETE *********

	% Begin delete procedure

	% Get handle to current figure
	figH=gcf;
	
	% Get links to objects
	links=get(figH,'UserData');

	ginput(1);
	
	gco;
	
	if figH~=gco,
		ud=get(gco,'UserData');
		if isempty(ud),
			delete(gco);
		elseif ud(1,1)~=1,
			delete(gco);
		else
			% do nothing - cannot delete tagged object
		end
	end

	% End delete procedure
	
elseif strcmp(action,'zoomin'),			% ********* CASE Zoom In *********

	% Begin Zoom In procedure

	% Get handle to current figure
	figH=gcf;

	% Get links to objects
	links=get(figH,'UserData');

	xlim=get(gca,'XLim');
	ylim=get(gca,'YLim');

	offset=[ xlim(1,1)+xlim(1,2), ylim(1,1)+ylim(1,2) ]./2;
	size=[ xlim(1,2)-xlim(1,1), ylim(1,2)-ylim(1,1) ]./2;

	set(gca,'XLim',[-size(1,1), size(1,1)]./2+offset(1,1));
	set(gca,'YLim',[-size(1,2), size(1,2)]./2+offset(1,2));


	% End Zoom In procedure

elseif strcmp(action,'nozoom'),			% ********* CASE Normal Zoom *********

	% Begin Normal Zoom procedure

	% Get handle to current figure
	figH=gcf;

	% Get links to objects
	links=get(figH,'UserData');

	set(gca,'XLim',[-1.15 1.15]);
	set(gca,'YLim',[-1.15 1.15]);

	% End No Zoom procedure

elseif strcmp(action,'zoomout'),		% ********* CASE Zoom Out *********

	% Begin Zoom Out procedure

	% Get handle to current figure
	figH=gcf;

	% Get links to objects
	links=get(figH,'UserData');

	xlim=get(gca,'XLim');
	ylim=get(gca,'YLim');

	offset=[ xlim(1,1)+xlim(1,2), ylim(1,1)+ylim(1,2) ]./2;
	size=[ xlim(1,2)-xlim(1,1), ylim(1,2)-ylim(1,1) ]./2;

	set(gca,'XLim',2.*[-size(1,1), size(1,1)]+offset(1,1));
	set(gca,'YLim',2.*[-size(1,2), size(1,2)]+offset(1,2));


	% End Zoom Out procedure
	
elseif strcmp(action,'offset'),			% ********* CASE Offset *********

	% Begin offset procedure

	% Get handle to current figure
	figH=gcf;

	% Get links to objects
	links=get(figH,'UserData');

	newoffset=ginput(1);

	xlim=get(gca,'XLim');
	ylim=get(gca,'YLim');

	% Check if selected position within plot before moving the origin
   	insideplot=(newoffset(1,1)>xlim(1,1)) &...
    			(newoffset(1,1)<xlim(1,2)) &...
    			(newoffset(1,2)>ylim(1,1)) &...
			(newoffset(1,2)<ylim(1,2));
    
   	if insideplot,

		oldoffset=[ (xlim(1,1)+xlim(1,2))/2, (ylim(1,1)+ylim(1,2))/2 ];
		xlim=xlim-oldoffset(1,1)+newoffset(1,1);
		ylim=ylim-oldoffset(1,2)+newoffset(1,2);

		set(gca,'XLim',xlim);
		set(gca,'YLim',ylim);

	end

	% End offset procedure
	
elseif strcmp(action,'placetext'),		% ********* CASE Place Text *********

	% Begin Place Text procedure

	% Get handle to current figure
	figH=gcf;

	% Get links to objects
	links=get(figH,'UserData');

	% Get starting point for the text in the edit field 
	Pt=ginput(1);
	text(Pt(1,1),Pt(1,2),get(links(editTextHindex),'String'));

	% End Place Text procedure
	
elseif strcmp(action,'done'),			% ********* CASE CLOSE *********

	% Begin close procedure

    close
    
	% End close procedure
    
end     % End of long "case action of..." statement.
